//
// modified of sample source Hadr00
// Ryuki Tanaka
//
#include "DetectorConstruction.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

#include "HimeSD.hh"
#include "G4SDManager.hh"
#include "G4VSensitiveDetector.hh"

#include <sstream>
#include <algorithm>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorConstruction::DetectorConstruction()
{
  logicTarget = 0;
  logicTargetMother = 0;
  logicTargetGrandMother = 0;
  logicWorld  = 0;
  himeSD = 0;

  targetMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");
  targetmotherMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
  targetgrandmotherMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
  worldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorConstruction::~DetectorConstruction()
{}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  G4int targetNumber(10);
  G4int targetmotherNumber(5);
  G4ThreeVector targetSize(40.0*cm, 4.0*cm, 2.0*cm);

  // Target LV
  G4ThreeVector target_pos(0., -targetSize.y()*targetNumber/2.0 + targetSize.y()/2.0, -targetSize.z()/2.0) ;
  G4Box* solidT = new G4Box("Target",
			    targetSize.x()/2.,
			    targetSize.y()/2.,
			    targetSize.z()/2.);
  logicTarget = new G4LogicalVolume( solidT,targetMaterial,"Target");

  // Target Mother LV
  G4ThreeVector targetmother_size(targetSize.x(),
				  targetSize.y()*targetNumber,
				  2.*targetSize.z());
  G4ThreeVector targetmother_pos(0., 0., -targetmother_size.z()*targetmotherNumber/2.0 + targetmother_size.z()/2.0) ;
  G4Box* solidTM = new G4Box("TargetMother",
			     targetmother_size.x()/2.,
			     targetmother_size.y()/2.,
			     targetmother_size.z()/2.);
  logicTargetMother = new G4LogicalVolume(solidTM,targetmotherMaterial,"TargetMother");

  // Target Grand-Mother LV
  G4double longSide = std::max(targetmother_size.x(), targetmother_size.y());
  G4ThreeVector targetgrandmother_size(longSide,
				       longSide, 
				       targetmother_size.z()*targetmotherNumber);
  G4ThreeVector targetgrandmother_pos(targetgrandmother_size.x()/2.0+0.1*mm,
				      0.+0.1*mm,
				      8.*m+targetgrandmother_size.z()/2.0) ;
  G4Box* solidTGM = new G4Box("TargetGrandMother",
			      targetgrandmother_size.x()/2.,
			      targetgrandmother_size.y()/2.,
			      targetgrandmother_size.z()/2.);
  logicTargetGrandMother = new G4LogicalVolume(solidTGM,targetgrandmotherMaterial,"TargetGrandMother");

  // World LV
  G4ThreeVector world_size(1.2*targetgrandmother_size.x()*2.,
			   1.2*targetgrandmother_size.y(),
			   1.2*2.*(targetgrandmother_pos.z()+targetgrandmother_size.z()/2.0));
  G4Box* solidW = new G4Box("World",
			    world_size.x()/2.,
			    world_size.y()/2.,
			    world_size.z()/2.) ;
  logicWorld = new G4LogicalVolume(solidW,worldMaterial,"World");
  
  // Target PV
  G4String target_name("Target");
  G4ThreeVector target_pos_shift(0., targetSize.y(), 0.);
  for(G4int i = 0; i < targetNumber; ++i){
    std::ostringstream os;
    os << target_name << i;
    new G4PVPlacement(0,
		      target_pos,
		      logicTarget,
		      os.str(),
		      logicTargetMother,
		      false,
		      i);
    target_pos += target_pos_shift;
  }

  // Target Mother PV
  G4String targetmother_name("TargetMother");
  G4ThreeVector targetmother_pos_shift(0., 0., targetmother_size.z());
  for(G4int i = 0; i < targetmotherNumber; ++i){
    G4RotationMatrix armRot= G4RotationMatrix();
    armRot.rotateZ(-90.*deg*(i%2));
    std::ostringstream os;
    os << targetmother_name << i;
    new G4PVPlacement(G4Transform3D(armRot, targetmother_pos),
		      logicTargetMother,
		      os.str(),
		      logicTargetGrandMother,
		      false,
		      i);
    targetmother_pos += targetmother_pos_shift;
  }

  // Target Grand-Mother PV
  new G4PVPlacement(0,
		    targetgrandmother_pos,
		    logicTargetGrandMother,
		    "TargetGrandMother",
		    logicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector world_pos(0., 0., 0.) ;  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       world_pos,
					       logicWorld,
					       "World",
					       0,
					       false,
					       0);

  // add SD
  if(!himeSD){
    G4SDManager* SDman = G4SDManager::GetSDMpointer();
    himeSD = new HimeSD("/HimeSD");
    SDman->AddNewDetector(himeSD);
  }
  logicTarget->SetSensitiveDetector(himeSD);

  // colors
  logicWorld->SetVisAttributes(G4VisAttributes::Invisible);
  logicTargetMother->SetVisAttributes(G4VisAttributes::Invisible);
  logicTargetGrandMother->SetVisAttributes(G4VisAttributes::Invisible);

  G4VisAttributes* regCcolor = new G4VisAttributes(G4Colour(0., 1.0, 1.0));
  logicTarget->SetVisAttributes(regCcolor);

  // Print status
  G4cout << "### Target consist of " 
         << targetMaterial->GetName() 
         <<  "  ###" << G4endl;

  G4cout << *(G4Material::GetMaterialTable()) << G4endl;

  return world;
}
